import { inject, injectable } from "tsyringe";
import { getLocationForWifiMemoized } from "../proxy/wigle";
import { WifiScanRepository } from "../repositories/wifiScanRepository";

interface CreateWifiScanParams {
  lp_ttn_end_device_uplinks_id: string;
  mac: string;
  rssi: number;
}

interface UpdateWifiScanParams {
  wifi_scan_id: string;
  mac?: string;
  rssi?: number;
  latitude?: number;
  longitude?: number;
}

@injectable()
export class WifiScanService {
  constructor(
    @inject(WifiScanRepository) private repository: WifiScanRepository
  ) {}

  public async getAllWifiScans() {
    return this.repository.findAll();
  }

  public async getWifiScanById(id: string) {
    return this.repository.findById(id);
  }

  public async createWifiScan(data: CreateWifiScanParams) {
    const apiResponse = await getLocationForWifiMemoized(data.mac);

    if (apiResponse !== undefined && apiResponse.results.length > 0)
      return this.repository.create({
        ...data,
        latitude: apiResponse.results[0].trilat,
        longitude: apiResponse.results[0].trilong,
      });
  }

  public async createWifiScans(data: CreateWifiScanParams[]) {
    let wifiScans = await Promise.all(
      data.map(async (wifi) => {
        return await this.createWifiScan(wifi);
      })
    );
    return wifiScans.filter((wifi) => wifi !== undefined);
  }

  public async updateWifiScan(data: UpdateWifiScanParams) {
    return this.repository.update(data.wifi_scan_id, data);
  }

  public async deleteWifiScan(id: string) {
    return this.repository.delete(id);
  }
}
